# from typing import Dict, List, Literal, Union

# from dataclasses import Field
# from datetime import datetime
# from decimal import Decimal
# from enum import IntEnum

# from fastapi_utils.api_model import APIModel
# from thanakan.models.bankcode import AnyBankCode, BankCode
# from thanakan.services.model.common import ProxyType, SlipData


# class StatusCode(IntEnum):
#     """
#     only known status codes
#     """

#     success = 1000


# class Status(APIModel):
#     code: Union[StatusCode, int]
#     description: str


# class CredentialsData(APIModel):
#     access_token: str
#     token_type: Literal["Bearer"] = "Bearer"
#     expires_in: int
#     expires_at: int


# class BaseResponse(APIModel):
#     status: Status
#     data: Dict


# class SCBCredentialsResponse(BaseResponse):
#     data: CredentialsData


# class QR30Data(APIModel):
#     qr_raw_data: str
#     qr_image: str  # seems like a base64 string


# class CreateQR30Response(BaseResponse):
#     data: QR30Data


# class VerifyResponse(BaseResponse):
#     data: SlipData


# class TransactionInquiryResponse(BaseResponse):
#     data: List[Dict]


# class WebhookBody(APIModel):
#     transaction_date_and_time: datetime = Field(alias='transactionDateandTime')
#     sending_bank_code: Union[BankCode, AnyBankCode]
#     payer_name: str
#     payer_proxy_type: ProxyType
#     payer_proxy_id: str
#     payer_account_number: str
#     receiving_bank_code: Union[BankCode, AnyBankCode]
#     payee_name: str
#     payee_proxy_type: ProxyType
#     payee_proxy_id: str
#     payee_account_number: str
#     amount: Decimal
#     currency_code: str  # wtf "764", thb?
#     transaction_id: str
#     transaction_type: str
#     bill_payment_ref1: str
#     bill_payment_ref2: str
#     bill_payment_ref3: str
#     channel_code: str

#     class Config:
#         schema_extra = {
#             "example": {
#                 "transactionDateandTime": "2021-02-24T11:10:12+07:00",
#                 "sendingBankCode": "014",
#                 "payerName": "Susophida Khunnawut",
#                 "payerProxyType": "ACCOUNT",
#                 "payerProxyId": "1502830001",
#                 "payerAccountNumber": "1502830001",
#                 "receivingBankCode": "014",
#                 "payeeName": "TestBiller1608347078",
#                 "payeeProxyType": "BILLERID",
#                 "payeeProxyId": "925244165291982",
#                 "payeeAccountNumber": "0987654321",
#                 "amount": "100",
#                 "currencyCode": "764",
#                 "transactionId": "9ed5caf2-0a38-4f5a-80b1-20e76f26b2a6",
#                 "transactionType": "Domestic Transfers",
#                 "billPaymentRef1": "ABCDEFGHIJ1234567890",
#                 "billPaymentRef2": "ABCDEFGHIJ1234567890",
#                 "billPaymentRef3": "YQH",
#                 "channelCode": "PMH",
#             }
#         }



"""
ref. https://developer.scb/assets/documents/documentation/qr-payment/extracting-data-from-mini-qr.pdf
beware! the "Sample data on mini QR" is poison, there is no such structure in real world.
"""
from typing import List

from dataclasses import dataclass
from enum import Enum

from crccheck.crc import Crc16CcittFalse
from PIL import Image
from pydantic import BaseModel
from pyzbar.pyzbar import Decoded, decode
from pyzbar.wrapper import ZBarSymbol


class Tag(Enum):
    payload = "00"
    country_code = "51"
    crc = "91"


class SubTag(Enum):
    api_id = "00"
    sending_bank_id = "01"
    transaction_ref_id = "02"


@dataclass
class CodeSection:
    code: str
    is_under_payload: bool = False

    @property
    def tag(self):
        return self.code[:2]

    @property
    def tag_type(self):
        return SubTag(self.tag) if self.is_under_payload else Tag(self.tag)

    @property
    def length(self):
        return int(self.code[2:4])

    @property
    def data(self):
        return self.code[4 : 4 + self.length]

    @property
    def rest(self):
        r = self.code[4 + self.length :]
        if r:
            return r
        else:
            return None


class QrPayload(BaseModel):
    api_id: str
    sending_bank_id: str
    transaction_ref_id: str

    @classmethod
    def create_from_code(cls, code):
        api_id = CodeSection(code, is_under_payload=True)
        assert api_id.tag_type == SubTag.api_id

        sending_bank_id = CodeSection(api_id.rest, is_under_payload=True)
        assert sending_bank_id.tag_type == SubTag.sending_bank_id

        transaction_ref_id = CodeSection(
            sending_bank_id.rest, is_under_payload=True
        )
        assert transaction_ref_id.tag_type == SubTag.transaction_ref_id
        assert transaction_ref_id.rest is None

        return cls(
            api_id=api_id.data,
            sending_bank_id=sending_bank_id.data,
            transaction_ref_id=transaction_ref_id.data,
        )


class not_bank_slip(BaseException):
    pass


class QRData(BaseModel):
    payload: QrPayload
    country_code: str
    crc: str

    @classmethod
    def create_from_code(cls, code: str):
        code = code.strip()

        payload = CodeSection(code=code)
        assert payload.tag_type == Tag.payload

        country_code = CodeSection(code=payload.rest)
        assert country_code.tag_type == Tag.country_code

        crc = CodeSection(code=country_code.rest)
        assert crc.tag_type == Tag.crc
        assert crc.rest is None

        data_part = code[: -crc.length]
        calc_crc = Crc16CcittFalse.calchex(
            data_part.encode(encoding="ascii")
        ).upper()
        assert calc_crc == crc.data

        return cls(
            payload=QrPayload.create_from_code(payload.data),
            country_code=country_code.data,
            crc=crc.data,
        )

    @classmethod
    def create_from_image(cls, pil_image: Image):
        qr_inside: List[Decoded] = decode(
            pil_image, symbols=[ZBarSymbol.QRCODE]
        )
        if len(qr_inside) != 1:
            raise not_bank_slip("Expect only 1 qr in image.")

        qr_data = qr_inside[0].data.decode("UTF-8")

        return cls.create_from_code(qr_data)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['thanakan',
 'thanakan.models',
 'thanakan.services',
 'thanakan.services.model',
 'thanakan.slip']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=8.1.0,<9.0.0',
 'crccheck>=1.0,<2.0',
 'fastapi-utils>=0.2.1,<0.3.0',
 'furl>=2.1.0,<3.0.0',
 'google-cloud-documentai>=0.3.0,<0.4.0',
 'google-cloud-secret-manager>=2.7.2,<3.0.0',
 'httpx-auth>=0.8.0,<0.9.0',
 'httpx[http2]>=0.16.1,<0.17.0',
 'locate>=0.0.1,<0.0.2',
 'loguru>=0.5.3,<0.6.0',
 'onepasswordconnectsdk>=1.1.0,<2.0.0',
 'parse-with-dot-access>=1.18.0,<2.0.0',
 'pydantic>=1.7.3,<2.0.0',
 'pyzbar-x>=0.2.1,<0.3.0']

setup_kwargs = {
    'name': 'thanakan',
    'version': '0.1.3',
    'description': 'Awesome `thanakan` is a Python cli/package created with https://github.com/TezRomacH/python-package-template',
    'long_description': '# thanakan\nPython Interface for Thai Bank API, KBANK, SCB, QR Code and slip verification\n\n## Pre-Requisit \n```\nsudo apt-get install libzbar0\n```\nmore on [pyzbar](https://pypi.org/project/pyzbar/)\n',
    'author': 'codustry',
    'author_email': 'hello@codustry.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/codustry/thanakan',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Literal, Optional, Union

from decimal import Decimal
from enum import auto

from fastapi_utils.api_model import APIModel
from fastapi_utils.enums import StrEnum
from pydantic import constr, root_validator, validator
from thanakan.models.bankcode import AnyBankCode, BankCode


class ProxyType(StrEnum):
    NATID = auto()
    MSISDN = auto()
    EWALLETID = auto()
    EMAIL = auto()
    BILLERID = auto()

    # SCB
    ACCOUNT = auto()


class Proxy(APIModel):
    type: Optional[ProxyType]
    value: Optional[str]

    @validator("*", pre=True)
    def convert_empty_str_to_none(cls, v):
        if isinstance(v, str) and v == "":
            return None
        else:
            return v


class AccountType(StrEnum):
    BANKAC = auto()
    TOKEN = auto()
    DUMMY = auto()


class Account(APIModel):
    type: Optional[AccountType]
    value: Optional[str]

    @validator("*", pre=True)
    def convert_empty_str_to_none(cls, v):
        if isinstance(v, str) and v == "":
            return None
        else:
            return v


class End(APIModel):
    display_name: Optional[str]
    name: Optional[str]
    proxy: Optional[Proxy]
    account: Optional[Account]

    @validator("proxy", "account")
    def dict_of_all_none_to_none(cls, v):
        if all((y is None for x, y in v)):
            return None
        else:
            return v


class SlipData(APIModel):
    language: Optional[Literal["EN", "TH"]]
    receiving_bank: Optional[Union[BankCode, AnyBankCode]]
    sending_bank: Union[BankCode, AnyBankCode]
    trans_ref: str
    trans_date: str
    trans_time: str
    sender: End
    receiver: End
    amount: Decimal
    paid_local_amount: Optional[Decimal]
    country_code: Optional[constr(min_length=2, max_length=2)]
    trans_fee_amount: Optional[str]
    ref1: Optional[str]
    ref2: Optional[str]
    ref3: Optional[str]
    to_merchant_id: Optional[str]

    @validator("*", pre=True)
    def convert_empty_str_to_none(cls, v):
        if isinstance(v, str) and v == "":
            return None
        else:
            return v
